@@ -3,27 +3,22 @@ from lamson.mail import MailRequest
 from lamson import view
 import os
 import time
-import shutil
 from app.model import post
 import jinja2
 import config
+import shutil
 
 view.LOADER = jinja2.Environment(loader=jinja2.PackageLoader('app', 'templates'))
-user = "test_user@test.com"
+user = "test_user@localhost"
 blog = "test_blog"
 name = "Tester Joe"
 
-def setup():
-    shutil.rmtree("app/data/posts")
-    os.mkdir("app/data/posts")
-
-
 def test_post():
     message = MailRequest("fakepeer", user,
-                          "%s@osb.test.com" % blog, "Fake body")
+                          "%s@oneshotblog.com" % blog, "Fake body")
     message['Subject'] = 'Test subject'
 
-    post.post(blog, user, message)
+    post.post(blog, user, "localhost", message)
 
     assert post.user_exists(user), "User dir not created."
     assert os.path.exists(post.blog_file_name(blog, user)), "File not made."
@@ -44,12 +39,12 @@ def test_make_user_dir():
 
 def test_remove_from_queue():
     message = MailRequest("fakepeer", user,
-                          "%s@osb.test.com" % blog, "Fake body")
+                          "%s@oneshotblog.com" % blog, "Fake body")
     message['Subject'] = 'Test subject'
 
     post_q = post.get_user_post_queue(post.get_user_dir(user))
 
-    post.post(blog, user, message)
+    post.post(blog, user, 'localhost', message)
 
     assert post_q.count(), "No messages in the post queue."
     count = post_q.count()
@@ -70,9 +65,4 @@ def test_blog_file_name():
     name = post.blog_file_name(blog, user)
     assert name.endswith("html")
 
-def test_textualize():
-    # try some ascii
-    txt = post.textilize("This is *hot*.")
-    assert txt
-
 
