@@ -1,3 +1,4 @@
+
 """
 The meat of Lamson, doing all the work that actually takes an email and makes
 sure that your code gets it.
@@ -24,6 +25,29 @@ after a @route or @route_like.
 
 If at any time you need to debug your routing setup just use the lamson routes
 command.
+
+Routing Control
+===============
+
+To control routing there are a set of decorators that you apply to your
+functions.
+
+* @route -- The main routing function that determines what addresses you are
+interested in.
+* @route_like -- Says that this function routes like another one.
+* @stateless -- Indicates this function always runs on each route encountered, and
+no state is maintained.
+* @nolocking -- Use this if you want this handler to run parallel without any
+locking around Lamson internals.  SUPER DANGEROUS, add @stateless as well.
+* @state_key_generator -- Used on a function that knows how to make your state
+keys for the module, for example if module_name + message.route_to is needed to maintain
+state.
+
+It's best to put @route or @route_like as the first decorator, then the others 
+after that.
+
+The @state_key_generator is different since it's not intended to go on a handler
+but instead on a simple function, so it shouldn't be combined with the others.
 """
 
 from __future__ import with_statement
@@ -31,19 +55,20 @@ from functools import wraps
 import re
 import logging
 import sys
-import email.utils
 import shelve
 import threading
 
 ROUTE_FIRST_STATE = 'START'
 LOG = logging.getLogger("routing")
+DEFAULT_STATE_KEY = lambda mod, msg: mod
+
 
 class StateStorage(object):
     """
     The base storage class you need to implement for a custom storage
     system.
     """
-    def get(self, module, sender):
+    def get(self, key, sender):
         """
         You must implement this so that it returns a single string
         of either the state for this combination of arguments, OR
@@ -51,7 +76,7 @@ class StateStorage(object):
         """
         raise NotImplementedError("You have to implement a StateStorage.get.")
 
-    def set(self, module, sender, state):
+    def set(self, key, sender, state):
         """
         Set should take the given parameters and consistently set the state for 
         that combination such that when StateStorage.get is called it gives back
@@ -76,20 +101,25 @@ class MemoryStorage(StateStorage):
     def __init__(self):
         self.states = {}
 
-    def get(self, module, sender):
-        key = self.key(module, sender)
+    def get(self, key, sender):
+        key = self.key(key, sender)
         try:
             return self.states[key]
         except KeyError:
-            self.set(module, sender, ROUTE_FIRST_STATE)
             return ROUTE_FIRST_STATE
 
-    def set(self, module, sender, state):
-        key = self.key(module, sender)
-        self.states[key] = state
+    def set(self, key, sender, state):
+        key = self.key(key, sender)
+        if state == ROUTE_FIRST_STATE:
+            try:
+                del self.states[key]
+            except KeyError:
+                pass
+        else:
+            self.states[key] = state
 
-    def key(self, module, sender):
-        return repr([module, sender])
+    def key(self, key, sender):
+        return repr([key, sender])
 
     def clear(self):
         self.states.clear()
@@ -102,35 +132,34 @@ class ShelveStorage(MemoryStorage):
     install if you need to persist your states (most likely), but if you 
     have a database, you'll need to write your own StateStorage that 
     uses your ORM or database to store.  Consider this an example.
+
+    NOTE: Because of shelve limitations you can only use ASCII encoded keys.
     """
     def __init__(self, database_path):
         """Database path depends on the backing library use by Python's shelve."""
         self.database_path = database_path
         self.lock = threading.RLock()
 
-    def get(self, module, sender):
+    def get(self, key, sender):
         """
         This will lock the internal thread lock, and then retrieve from the
-        shelf whatever module you request.  If the module is not found then it
+        shelf whatever key you request.  If the key is not found then it
         will set (atomically) to ROUTE_FIRST_STATE.
         """
         with self.lock:
-            store = shelve.open(self.database_path)
-            try:
-                key = store[self.key(module, sender)]
-            except KeyError:
-                self.set(module, sender, ROUTE_FIRST_STATE)
-                key = ROUTE_FIRST_STATE
-            return key
+            self.states = shelve.open(self.database_path)
+            value = super(ShelveStorage, self).get(key.encode('ascii'), sender)
+            self.states.close()
+            return value
 
-    def set(self, module, sender, state):
+    def set(self, key, sender, state):
         """
         Acquires the self.lock and then sets the requested state in the shelf.
         """
         with self.lock:
-            store = shelve.open(self.database_path)
-            store[self.key(module, sender)] = state
-            store.close()
+            self.states = shelve.open(self.database_path)
+            super(ShelveStorage, self).set(key.encode('ascii'), sender, state)
+            self.states.close()
 
     def clear(self):
         """
@@ -138,9 +167,9 @@ class ShelveStorage(MemoryStorage):
         states are clear.  In production this could be a bad thing.
         """
         with self.lock:
-            store = shelve.open(self.database_path)
-            store.clear()
-            store.close()
+            self.states = shelve.open(self.database_path)
+            super(ShelveStorage, self).clear()
+            self.states.close()
 
 
 
@@ -177,6 +206,9 @@ class RoutingBase(object):
     running.
 
     It's your job to keep things straight if you do that.
+
+    NOTE: See @state_key_generator for a way to change what the key is to 
+    STATE_STORE for different state control options.
     """
 
     def __init__(self):
@@ -184,8 +216,9 @@ class RoutingBase(object):
         self.ORDER = []
         self.DEFAULT_CAPTURES = {}
         self.STATE_STORE = MemoryStorage()
-        self.HANDLERS = set()
+        self.HANDLERS = {}
         self.RELOAD = False
+        self.LOG_EXCEPTIONS = True
         self.UNDELIVERABLE_QUEUE = None
         self.lock = threading.RLock()
         self.call_lock = threading.RLock()
@@ -201,19 +234,17 @@ class RoutingBase(object):
                 self.REGISTERED[format][1].append(func)
             else:
                 self.ORDER.append(format)
-                self.REGISTERED[format] = (re.compile(format), [func])
+                self.REGISTERED[format] = (re.compile(format, re.IGNORECASE), [func])
 
     def match(self, address):
         """
         This is a generator that goes through all the routes and
-        yields each match it finds.  It only matches on the 
-        address portion of "First Last" <address> formatted
-        addresses.
+        yields each match it finds.  It expects you to give it a
+        blah@blah.com address, NOT "Joe Blow" <blah@blah.com>.
         """
-        name, real_address = email.utils.parseaddr(address)
         for format in self.ORDER:
             regex, functions = self.REGISTERED[format]
-            match = regex.match(real_address)
+            match = regex.match(address)
             if match:
                 yield functions, match.groupdict()
 
@@ -234,8 +265,8 @@ class RoutingBase(object):
 
     def get_state(self, module_name, message):
         """Returns the state that this module is in for the given message (using its from)."""
-        _, address = email.utils.parseaddr(message['from'])
-        return self.STATE_STORE.get(module_name, address)
+        key = self.state_key(module_name, message)
+        return self.STATE_STORE.get(key, message.route_from)
 
     
     def in_state(self, func, message):
@@ -255,15 +286,44 @@ class RoutingBase(object):
         state = self.get_state(func.__module__, message)
         return state and state == 'ERROR'
 
+    def state_key(self, module_name, message):
+        """
+        Given a module_name we need to get a state key for, and a
+        message that has information to make the key, this function
+        calls any registered @state_key_generator and returns that
+        as the key.  If none is given then it just returns module_name
+        as the key.
+        """
+        key_func = self.HANDLERS.get(module_name, DEFAULT_STATE_KEY)
+        return key_func(module_name, message)
 
     def set_state(self, module_name, message, state):
         """
         Sets the state of the given module (a string) according to the message to the requested
         state (a string).  This is also how you can force another FSM to a required state.
         """
-        _, address = email.utils.parseaddr(message['from'])
-        self.STATE_STORE.set(module_name, address, state)
+        key = self.state_key(module_name, message)
+        self.STATE_STORE.set(key, message.route_from, state)
 
+    def _collect_matches(self, message, route_to):
+        in_state_found = False
+
+        for functions, matchkw in self.match(route_to):
+            for func in functions:
+                if lamson_setting(func, 'stateless'):
+                    yield func, matchkw
+                elif not in_state_found and self.in_state(func, message):
+                    in_state_found = True
+                    yield func, matchkw
+
+    def _enqueue_undeliverable(self, message):
+        if self.UNDELIVERABLE_QUEUE:
+            LOG.debug("Message to %r from %r undeliverable, putting in undeliverable queue (# of recipients: %d).",
+                      message.route_to, message.route_from, len(message.route_to))
+            self.UNDELIVERABLE_QUEUE.push(message)
+        else:
+            LOG.debug("Message to %r from %r didn't match any handlers. (# recipients: %d)",
+                      message.route_to, message.route_from, len(message.route_to))
 
     def deliver(self, message):
         """
@@ -288,34 +348,20 @@ class RoutingBase(object):
 
         called_count = 0
 
-        for functions, matchkw in self.match(message['to']):
-            to_call = []
-            in_state_found = False
+        for routing_on in message.route_to:
+            for func, matchkw in self._collect_matches(message, routing_on):
+                LOG.debug("Matched %r against %s.", routing_on, func.__name__)
 
-            for func in functions:
-                if lamson_setting(func, 'stateless'):
-                    to_call.append(func)
-                elif not in_state_found and self.in_state(func, message):
-                    to_call.append(func)
-                    in_state_found = True
-
-            called_count += len(to_call)
-
-            for func in to_call:
                 if lamson_setting(func, 'nolocking'):
                     self.call_safely(func, message,  matchkw)
                 else:
                     with self.call_lock:
                         self.call_safely(func, message, matchkw)
 
+                called_count += 1
+
         if called_count == 0:
-            if self.UNDELIVERABLE_QUEUE:
-                LOG.debug("Message to %r from %r undeliverable, putting in undeliverable queue.",
-                          message['to'], message['from'])
-                self.UNDELIVERABLE_QUEUE.push(message)
-            else:
-                LOG.debug("Message to %r from %r didn't match any handlers.",
-                          message['to'], message['from'])
+            self._enqueue_undeliverable(message)
 
 
     def call_safely(self, func, message, kwargs):
@@ -323,14 +369,26 @@ class RoutingBase(object):
         Used by self to call a function and log exceptions rather than
         explode and crash.
         """
+        from lamson.server import SMTPError
+
         try:
             func(message, **kwargs)
             LOG.debug("Message to %s was handled by %s.%s",
-                          message['to'], func.__module__, func.__name__)
+                          message.route_to, func.__module__, func.__name__)
+        except SMTPError:
+            raise
         except:
-            LOG.exception("!!! ERROR handling %s.%s", func.__module__, func.__name__)
-            self.set_state(func.__module__, message, 'ERROR')
-            if self.UNDELIVERABLE_QUEUE: self.UNDELIVERABLE_QUEUE.push(message)
+            if 'ERROR' in dir(sys.modules[func.__module__]):
+                self.set_state(func.__module__, message, 'ERROR')
+
+            if self.UNDELIVERABLE_QUEUE:
+                self.UNDELIVERABLE_QUEUE.push(message)
+
+            if self.LOG_EXCEPTIONS:
+                LOG.exception("!!! ERROR handling %s.%s", func.__module__, func.__name__)
+            else:
+                raise
+
 
     def clear_states(self):
         """Clears out the states for unit testing."""
@@ -351,12 +409,19 @@ class RoutingBase(object):
         listed.
         """
         with self.lock:
-            self.HANDLERS.update(handlers)
-            for module in self.HANDLERS:
+            for module in handlers:
                 try:
                     __import__(module, globals(), locals())
+
+                    if module not in self.HANDLERS:
+                        # they didn't specify a key generator, so use the
+                        # default one for now
+                        self.HANDLERS[module] = DEFAULT_STATE_KEY
                 except:
-                    LOG.exception("ERROR IMPORTING %r MODULE:" % module)
+                    if self.LOG_EXCEPTIONS:
+                        LOG.exception("ERROR IMPORTING %r MODULE:" % module)
+                    else:
+                        raise
 
     def reload(self):
         """
@@ -370,7 +435,10 @@ class RoutingBase(object):
                     try:
                         reload(sys.modules[module])
                     except:
-                        LOG.exception("ERROR RELOADING %r MODULE:" % module)
+                        if self.LOG_EXCEPTIONS:
+                            LOG.exception("ERROR RELOADING %r MODULE:" % module)
+                        else:
+                            raise
 
 Router = RoutingBase()
 
@@ -431,7 +499,7 @@ class route(object):
                 next_state = func(message, *args, **kw)
 
                 if next_state:
-                    Router.set_state(func.__module__, message, next_state.__name__)
+                    Router.set_state(next_state.__module__, message, next_state.__name__)
 
         Router.register_route(self.format, routing_wrapper)
         return routing_wrapper
@@ -524,4 +592,12 @@ def nolocking(func):
     func._lamson_settings['nolocking'] = True
     return func
 
-
+def state_key_generator(func):
+    """
+    Used to indicate that a function in your handlers should be used
+    to determine what they key is for state storage.  It should be a 
+    function that takes the module_name and message being worked on
+    and returns a string.
+    """
+    Router.HANDLERS[func.__module__] = func
+    return func
