@@ -1,7 +1,11 @@
 from lamson.routing import *
 
+@state_key_generator
+def simple_key_gen(module_name, message):
+    return module_name
+
 # common routing capture regexes go in here, you can override them in @route
-Router.defaults(host="test.com", 
+Router.defaults(host="localhost", 
                 action="[a-zA-Z0-9]+",
                 list_name="[a-zA-Z.0-9]+")
 
@@ -29,6 +33,9 @@ def NEXT(message, list_name=None, action=None, host=None):
     print "NEXT", message, list_name, action, host
     return END
 
+def ERROR(message):
+    return ERROR
+
 @route("(anything)@(host)", anything=".*")
 def END(message, anything=None, host=None):
     print "END", anything, host
